--[[-------------------------------------------------------------
SineLab                                                         -
Merge Sine waves together                                       -
]]---------------------------------------------------------------

VERSION = "0.01"
AUTHOR = "martblek (martblek@gmail.com)"
DEBUG = false

CONFIG = renoise.Document.create("Macuilxochitl"){
  info="Macuilxochitl patch file",
  version=VERSION,
  author=AUTHOR,
  name = "Default_Config",
  sample = {
    frames=1024,
    frequency1=1.0,
    frequency2=3.0,
    frequency3=5.0,
    amplitude1=0.1,
    amplitude2=0.15,
    amplitude3=0.5,
    phase1=0.5,
    phase2=0.0,
    phase3=0.1,
  },
  noise = {
    on_off=false,
    amount=0.0,
  },
  gaussian = {
    on_off=false,
    v1=0.3,
    v2=0.8,
    v3=0.4,
    v4=0.1,
    v5=0.2,
    n1=1.0,
    n2=2.0,
    n3=3.0,
    n4=4.0,
    n5=5.0,
    s1=-3.0,
    s2=-2.7,
    s3=-7.0,
    s4=-5.2,
    s5=-1.4,
  },
  operator={
    op1=1,
    op2=2,
  }
}

--CONFIG:save_as("patches/default.mxp")

SAMPLE_FREQUENCY = 44100
SAMPLE_BIT_DEPTH = 32
SAMPLE_CHANNELS = 1
SAMPLE_FRAMES = CONFIG.sample.frames.value

SAMPLE_FREQUENCY_1 = CONFIG.sample.frequency1.value
SAMPLE_FREQUENCY_2 = CONFIG.sample.frequency2.value
SAMPLE_FREQUENCY_3 = CONFIG.sample.frequency3.value

SAMPLE_AMPLITUDE_1 = CONFIG.sample.amplitude1.value
SAMPLE_AMPLITUDE_2 = CONFIG.sample.amplitude2.value
SAMPLE_AMPLITUDE_3 = CONFIG.sample.amplitude3.value

SAMPLE_PHASE_1 = CONFIG.sample.phase1.value
SAMPLE_PHASE_2 = CONFIG.sample.phase2.value
SAMPLE_PHASE_3 = CONFIG.sample.phase3.value

SAMPLE_MAX_FREQ = 1024.0
SAMPLE_MAX_AMPLITUDE = 1.0
SAMPLE_MIN_AMPLITUDE = -1.0
SAMPLE_MAX_PHASE = 10.0

SAMPLE_NOISE_BOOL = CONFIG.noise.on_off.value
SAMPLE_NOISE_AMOUNT = CONFIG.noise.amount.value
SAMPLE_MAX_NOISE = 9.99

GAUSSIAN_ON_OFF = CONFIG.gaussian.on_off.value
GAUSSIAN_V1 = CONFIG.gaussian.v1.value
GAUSSIAN_V2 = CONFIG.gaussian.v2.value
GAUSSIAN_V3 = CONFIG.gaussian.v3.value
GAUSSIAN_V4 = CONFIG.gaussian.v4.value
GAUSSIAN_V5 = CONFIG.gaussian.v5.value
GAUSSIAN_V_MAX = 1.0
GAUSSIAN_V_MIN = 0.0

GAUSSIAN_N1 = CONFIG.gaussian.n1.value
GAUSSIAN_N2 = CONFIG.gaussian.n2.value
GAUSSIAN_N3 = CONFIG.gaussian.n3.value
GAUSSIAN_N4 = CONFIG.gaussian.n4.value
GAUSSIAN_N5 = CONFIG.gaussian.n5.value
GAUSSIAN_N_MAX = 10.0
GAUSSIAN_N_MIN = 0.0

GAUSSIAN_S1 = CONFIG.gaussian.s1.value
GAUSSIAN_S2 = CONFIG.gaussian.s2.value
GAUSSIAN_S3 = CONFIG.gaussian.s3.value
GAUSSIAN_S4 = CONFIG.gaussian.s4.value
GAUSSIAN_S5 = CONFIG.gaussian.s5.value
GAUSSIAN_S_MAX = 0.0
GAUSSIAN_S_MIN = -10.0

OPERATIONS = {"Add", "Sub", "Mul", "Div"}
OP1 = tonumber(CONFIG.operator.op1.value)
OP2 = tonumber(CONFIG.operator.op2.value)

PATCHES = nil

vb = nil
dialog = nil

--[[-------------------------------------------------------------
gui constants                                                   -
]]---------------------------------------------------------------
DEFAULT_MARGIN = renoise.ViewBuilder.DEFAULT_CONTROL_MARGIN
DEFAULT_SPACING = renoise.ViewBuilder.DEFAULT_CONTROL_SPACING
DEFAULT_HEIGHT = renoise.ViewBuilder.DEFAULT_CONTROL_HEIGHT
MINISLIDER_HEIGHT = 12
SELECTED_PAGE = 3
MENU_BUTTON_WIDTH = 64
MENU_BUTTON_HEIGHT= 24


--[[------------------------------------------------------------- 
Register menu in renoise                                        -
]]---------------------------------------------------------------
renoise.tool():add_menu_entry{
  name = "Sample Editor: Macuilxochitl",
  invoke = function()
    show_gui()
  end
}

--[[-------------------------------------------------------------
Load patch file                                                 -
]]---------------------------------------------------------------
function load_patch_file(patchname)
  local result = CONFIG:load_from(patchname)
  if result ~= 0 then
    SAMPLE_FRAMES = CONFIG.sample.frames.value
    SAMPLE_FREQUENCY_1 = CONFIG.sample.frequency1.value
    SAMPLE_FREQUENCY_2 = CONFIG.sample.frequency2.value
    SAMPLE_FREQUENCY_3 = CONFIG.sample.frequency3.value
    SAMPLE_AMPLITUDE_1 = CONFIG.sample.amplitude1.value
    SAMPLE_AMPLITUDE_2 = CONFIG.sample.amplitude2.value
    SAMPLE_AMPLITUDE_3 = CONFIG.sample.amplitude3.value
    SAMPLE_PHASE_1     = CONFIG.sample.phase1.value
    SAMPLE_PHASE_2     = CONFIG.sample.phase2.value
    SAMPLE_PHASE_3     = CONFIG.sample.phase3.value
    SAMPLE_NOISE_BOOL  = CONFIG.noise.on_off.value
    SAMPLE_NOISE_AMOUNT= CONFIG.noise.amount.value
    GAUSSIAN_ON_OFF    = CONFIG.gaussian.on_off.value
    GAUSSIAN_V1        = CONFIG.gaussian.v1.value
    GAUSSIAN_V2        = CONFIG.gaussian.v2.value
    GAUSSIAN_V3        = CONFIG.gaussian.v3.value
    GAUSSIAN_V4        = CONFIG.gaussian.v4.value
    GAUSSIAN_V5        = CONFIG.gaussian.v5.value
    GAUSSIAN_N1        = CONFIG.gaussian.n1.value
    GAUSSIAN_N2        = CONFIG.gaussian.n2.value
    GAUSSIAN_N3        = CONFIG.gaussian.n3.value
    GAUSSIAN_N4        = CONFIG.gaussian.n4.value
    GAUSSIAN_N5        = CONFIG.gaussian.n5.value
    GAUSSIAN_S1        = CONFIG.gaussian.s1.value
    GAUSSIAN_S2        = CONFIG.gaussian.s2.value
    GAUSSIAN_S3        = CONFIG.gaussian.s3.value
    GAUSSIAN_S4        = CONFIG.gaussian.s4.value
    GAUSSIAN_S5        = CONFIG.gaussian.s5.value
    OP1                = CONFIG.operator.op1.value
    OP2                = CONFIG.operator.op2.value

    -- refresh gui widgets

    vb.views.freq1slider.value = SAMPLE_FREQUENCY_1
    vb.views.freq1box.value = SAMPLE_FREQUENCY_1
    vb.views.amp1slider.value = SAMPLE_AMPLITUDE_1
    vb.views.amp1box.value = SAMPLE_AMPLITUDE_1
    vb.views.phase1slider.value = SAMPLE_PHASE_1
    vb.views.phase1box.value = SAMPLE_PHASE_1

    vb.views.freq2slider.value = SAMPLE_FREQUENCY_2
    vb.views.freq2box.value = SAMPLE_FREQUENCY_2
    vb.views.amp2slider.value = SAMPLE_AMPLITUDE_2
    vb.views.amp2box.value = SAMPLE_AMPLITUDE_2
    vb.views.phase2slider.value = SAMPLE_PHASE_2
    vb.views.phase2box.value = SAMPLE_PHASE_2
    
    vb.views.freq3slider.value = SAMPLE_FREQUENCY_3
    vb.views.freq3box.value = SAMPLE_FREQUENCY_3
    vb.views.amp3slider.value = SAMPLE_AMPLITUDE_3
    vb.views.amp3box.value = SAMPLE_AMPLITUDE_3
    vb.views.phase3slider.value = SAMPLE_PHASE_3
    vb.views.phase3box.value = SAMPLE_PHASE_3

    vb.views.noise.value = SAMPLE_NOISE_AMOUNT
    if SAMPLE_NOISE_AMOUNT < 0.5 then
      vb.views.noisy_message.text = "Sak Nik sleeps"
    else
      vb.views.noisy_message.text = "Sak Nik is awake"
    end

    vb.views.sample_frames_slider.value = SAMPLE_FRAMES
    vb.views.sample_frames_box.value = SAMPLE_FRAMES

    vb.views.op1.value = OP1
    vb.views.op2.value = OP2

    if GAUSSIAN_ON_OFF then
      vb.views.gaussian_on_off.value = true
    else
      vb.views.gaussian_on_off.value = false
    end

    vb.views.gauss_v1.value = GAUSSIAN_V1
    vb.views.gauss_v2.value = GAUSSIAN_V2
    vb.views.gauss_v3.value = GAUSSIAN_V3
    vb.views.gauss_v4.value = GAUSSIAN_V4
    vb.views.gauss_v5.value = GAUSSIAN_V5
    vb.views.gauss_n1.value = GAUSSIAN_N1
    vb.views.gauss_n2.value = GAUSSIAN_N2
    vb.views.gauss_n3.value = GAUSSIAN_N3
    vb.views.gauss_n4.value = GAUSSIAN_N4
    vb.views.gauss_n5.value = GAUSSIAN_N5
    vb.views.gauss_s1.value = GAUSSIAN_S1
    vb.views.gauss_s2.value = GAUSSIAN_S2
    vb.views.gauss_s3.value = GAUSSIAN_S3
    vb.views.gauss_s4.value = GAUSSIAN_S4
    vb.views.gauss_s5.value = GAUSSIAN_S5
    
  else
    renoise.app():show_error("ERROR: Loading patch file "..patchname)
  end
end

--[[-------------------------------------------------------------
Save patch                                                      -
]]---------------------------------------------------------------
function save_patch_file(filename)
  --renoise.app():show_warning(filename)
  CONFIG.sample.frames.value = SAMPLE_FRAMES
  CONFIG.sample.frequency1.value = SAMPLE_FREQUENCY_1
  CONFIG.sample.frequency2.value = SAMPLE_FREQUENCY_2
  CONFIG.sample.frequency3.value = SAMPLE_FREQUENCY_3
  CONFIG.sample.amplitude1.value = SAMPLE_AMPLITUDE_1
  CONFIG.sample.amplitude2.value = SAMPLE_AMPLITUDE_2
  CONFIG.sample.amplitude3.value = SAMPLE_AMPLITUDE_3
  CONFIG.sample.phase1.value = SAMPLE_PHASE_1
  CONFIG.sample.phase2.value = SAMPLE_PHASE_2
  CONFIG.sample.phase3.value = SAMPLE_PHASE_3
  CONFIG.noise.on_off.value = SAMPLE_NOISE_BOOL
  CONFIG.noise.amount.value = SAMPLE_NOISE_AMOUNT
  CONFIG.gaussian.on_off.value = GAUSSIAN_ON_OFF
  CONFIG.gaussian.v1.value = GAUSSIAN_V1
  CONFIG.gaussian.v2.value = GAUSSIAN_V2
  CONFIG.gaussian.v3.value = GAUSSIAN_V3
  CONFIG.gaussian.v4.value = GAUSSIAN_V4
  CONFIG.gaussian.v5.value = GAUSSIAN_V5
  CONFIG.gaussian.n1.value = GAUSSIAN_N1
  CONFIG.gaussian.n2.value = GAUSSIAN_N2
  CONFIG.gaussian.n3.value = GAUSSIAN_N3
  CONFIG.gaussian.n4.value = GAUSSIAN_N4
  CONFIG.gaussian.n5.value = GAUSSIAN_N5
  CONFIG.gaussian.s1.value = GAUSSIAN_S1
  CONFIG.gaussian.s2.value = GAUSSIAN_S2
  CONFIG.gaussian.s3.value = GAUSSIAN_S3
  CONFIG.gaussian.s4.value = GAUSSIAN_S4
  CONFIG.gaussian.s5.value = GAUSSIAN_S5
  CONFIG.name.value = filename
  CONFIG.operator.op1.value = tonumber(math.floor(OP1))
  CONFIG.operator.op2.value = tonumber(math.floor(OP2))

  local result = CONFIG:save_as(filename)
end

--[[-------------------------------------------------------------
Maping function from one range to another range                 -
]]---------------------------------------------------------------
function map_from_to(val, fmin, fmax, tmin, tmax)
  return (val - fmin) / (fmax - fmin) * (tmax - tmin) + tmin
end

--[[-------------------------------------------------------------
Some functions for late use                                     -
]]---------------------------------------------------------------
function sign(x)
  -- return 0 or 1
  return x > 0 and 1 or (x == 0 and 0 or -1)
end

function sawtooth(x)
  -- Sawtooth wave with period 2pi
  return (x + math.pi) / math.pi % 2 - 1
end

function square(x)
  -- Square wave with period 2pi
  return sign(math.sin(x))
end

function gausian_mixture(value)

  local x1, x2, x3, x4, x5 = nil, nil, nil, nil, nil

  x1 = GAUSSIAN_V1 * math.exp(GAUSSIAN_S1 * (value - GAUSSIAN_N1) ^ 2)
  x2 = GAUSSIAN_V2 * math.exp(GAUSSIAN_S2 * (value - GAUSSIAN_N2) ^ 2)
  x3 = GAUSSIAN_V3 * math.exp(GAUSSIAN_S3 * (value - GAUSSIAN_N3) ^ 2)
  x4 = GAUSSIAN_V4 * math.exp(GAUSSIAN_S4 * (value - GAUSSIAN_N4) ^ 2)
  x5 = GAUSSIAN_V5 * math.exp(GAUSSIAN_S5 * (value - GAUSSIAN_N5) ^ 2)
  return x1 - x2 + x3 - x4 + x5

end

function generate_wavetable(length, f)
  local wavetable = {}
  for i=1, length do
    wavetable[i] = f(2 * math.pi * i / length)
  end
  return wavetable
end

--[[-------------------------------------------------------------
Paint magic :)                                                  -
]]---------------------------------------------------------------
function redraw_sample()

  if SAMPLE_AMPLITUDE_1 == 0 and SAMPLE_AMPLITUDE_2 == 0 and SAMPLE_AMPLITUDE_3 == 0 then
    -- prevent audio damage :)
    return
  end

  math.randomseed(os.time())

  local new_sample = nil
  local new_buffer = nil
  local instrument = renoise.song().selected_instrument

  --while table.getn(instrument.samples) > 0 do
    --  instrument:delete_sample_at(1)
    --end

    if table.getn(instrument.samples) < 1 then
      new_sample = instrument:insert_sample_at(1)
    else
      new_sample = instrument:sample(1)
    end

    local new_buffer = new_sample.sample_buffer

    -- if sample has same default values dont create new just rewrite
    if new_buffer.has_sample_data
      and new_buffer.number_of_frames == math.floor(SAMPLE_FRAMES)
      and new_buffer.sample_rate == SAMPLE_FREQUENCY
      and new_buffer.bit_depth == SAMPLE_BIT_DEPTH
      and new_buffer.number_of_channels == 1
    then
      --nope
    else
    -- create new shiny sample
      if
        SAMPLE_FRAMES > 0 and not new_buffer:create_sample_data(SAMPLE_FREQUENCY, SAMPLE_BIT_DEPTH, 1, SAMPLE_FRAMES)
      then
        renoise.app():show_error("Error during sample creation!")
          renoise.song():undo()
          return
        else
          new_sample.loop_mode=1
      end
    end

    new_buffer:prepare_sample_data_changes()

    local val_1 = nil
    local val_2 = nil
    local val_3 = nil
    local result = nil

    local max_value = -9999.0
    local min_value = 9999.0

    local arr = {}

    for frame_in_buffer=1, SAMPLE_FRAMES do
      
      -- calculate all three sin waves and sum
      val_1 = SAMPLE_AMPLITUDE_1 * math.sin(2.0 * math.pi * SAMPLE_FREQUENCY_1 * ((SAMPLE_PHASE_1 + frame_in_buffer) / SAMPLE_FRAMES))
      val_2 = SAMPLE_AMPLITUDE_2 * math.sin(2.0 * math.pi * SAMPLE_FREQUENCY_2 * ((SAMPLE_PHASE_2 + frame_in_buffer) / SAMPLE_FRAMES))
      val_3 = SAMPLE_AMPLITUDE_3 * math.sin(2.0 * math.pi * SAMPLE_FREQUENCY_3 * ((SAMPLE_PHASE_3 + frame_in_buffer) / SAMPLE_FRAMES))
      
      if OP1 == 1 then
        result = val_1 + val_2
      elseif OP1 == 2 then
        result = val_1 - val_2
      elseif OP1 == 3 then
        result = val_1 * val_2
      else
        result = val_1 / val_2
      end

      if OP2 == 1 then
        result = result + val_3
      elseif OP2 == 2 then
        result = result - val_3
      elseif OP2 == 3 then
        result = result * val_3
      else
        result = result / val_3
      end

      if SAMPLE_NOISE_BOOL then
        if math.random() > 0.5 then
          result = result + (math.random() / (SAMPLE_MAX_NOISE - SAMPLE_NOISE_AMOUNT +0.1))
        else
          result = result - (math.random() / (SAMPLE_MAX_NOISE - SAMPLE_NOISE_AMOUNT +0.1))
        end
      end

      if result > max_value then
        max_value = result
      end
      
      if result < min_value then
        min_value = result
      end

      arr[frame_in_buffer] = result

    end

    local test = {}
    local gauss = {}

    for i=1, #arr do
      test[i] = map_from_to(arr[i], min_value, max_value, -1, 1)
      if GAUSSIAN_ON_OFF then
        gauss[i] = test[i] * gausian_mixture(2 * math.pi * i / #arr)
        new_buffer:set_sample_data(1,i,gauss[i])
      else
        new_buffer:set_sample_data(1,i,test[i])
      end
    end

    new_buffer:finalize_sample_data_changes()
    new_sample.name = "Macuilxochitl Sample"
    instrument.name = "Macuilxochitl Instrument"

end

--[[-------------------------------------------------------------
Create whole tool gui                                           -
]]---------------------------------------------------------------
function show_gui()

  if (dialog and dialog.visible) then
    dialog:show()
    return
  end

  vb = renoise.ViewBuilder()

--[[----------------------------------------------------------------
Create OSC 1 settings                                              -
]]------------------------------------------------------------------
  local osc_1_gui = vb:column{
    style="group",
    margin=DEFAULT_MARGIN,
    spacing=DEFAULT_SPACING,
    height = DEFAULT_HEIGHT,
    vb:horizontal_aligner{
      mode="left",
      vb:text{
        text="Huehuecóyotl",
        font="bold",
        style="strong",
      },
    },
    -- --------------------------------------- frequency OSC 1
    vb:row{
      vb:text{text="Frequency", align="left", width=80},
      vb:minislider{
        height=MINISLIDER_HEIGHT,
        id = "freq1slider",
        width=120,
        min=0.001,
        max=SAMPLE_MAX_FREQ,
        value=SAMPLE_FREQUENCY_1,
        notifier = function(freq)
          SAMPLE_FREQUENCY_1 = tonumber(freq)
          vb.views.freq1box.value = tonumber(freq)
          redraw_sample()
        end
      },
      vb:valuefield{
        id="freq1box",
        width=50,
        min=0.001,
        max=SAMPLE_MAX_FREQ,
        value=SAMPLE_FREQUENCY_1,
        notifier = function(freq)
          SAMPLE_FREQUENCY_1 = tonumber(freq)
          vb.views.freq1slider.value = tonumber(freq)
          redraw_sample()
        end
      },
    },
    -- --------------------------------------- amplitude OSC 1
    vb:row{
      vb:text{text="Amplitude", align="left", width=80},
      vb:minislider{
        height=MINISLIDER_HEIGHT,
        id = "amp1slider",
        width=120,
        min=SAMPLE_MIN_AMPLITUDE,
        max=SAMPLE_MAX_AMPLITUDE,
        value=SAMPLE_AMPLITUDE_1,
        notifier = function(amplitude)
          SAMPLE_AMPLITUDE_1 = tonumber(amplitude)
          vb.views.amp1box.value = tonumber(amplitude)
          redraw_sample()
        end
      },
      vb:valuefield{
        id="amp1box",
        width=50,
        min=SAMPLE_MIN_AMPLITUDE,
        max=SAMPLE_MAX_AMPLITUDE,
        value=SAMPLE_AMPLITUDE_1,
        notifier = function(amplitude)
          SAMPLE_AMPLITUDE_1 = tonumber(amplitude)
          vb.views.amp1slider.value = tonumber(amplitude)
          redraw_sample()
        end
      },
    },
    -- ----------------------------------------- phase OSC 1
    vb:row{
      vb:text{text="Phase", align="left", width=80},
      vb:minislider{
        height=MINISLIDER_HEIGHT,
        id = "phase1slider",
        width=120,
        min=-SAMPLE_MAX_PHASE,
        max=SAMPLE_MAX_PHASE,
        value=SAMPLE_PHASE_1,
        notifier = function(phase)
          SAMPLE_PHASE_1 = tonumber(phase)
          vb.views.phase1box.value = tonumber(phase)
          redraw_sample()
        end
      },
      vb:valuefield{
        id="phase1box",
        width=50,
        min=-SAMPLE_MAX_PHASE,
        max=SAMPLE_MAX_PHASE,
        value=SAMPLE_PHASE_1,
        notifier = function(phase)
          SAMPLE_PHASE_1 = tonumber(phase)
          vb.views.phase1slider.value = tonumber(phase)
          redraw_sample()
        end
      },
    },
  }

--[[--------------------------------------------------------------
Oscilator 2 Settings                                             -
]]----------------------------------------------------------------
  local osc_2_gui = vb:column{
    style="group",
    margin=DEFAULT_MARGIN,
    vb:horizontal_aligner{
      mode="left",
      vb:text{
        text="Ah-Xoc-Xin",
        font="bold",
        style="strong",
      },
    },
    -- ---------------------------------------- frequency osc 2
    vb:row{
      vb:text{text="Frequency", align="left", width=80},
      vb:minislider{
        height=MINISLIDER_HEIGHT,
        id = "freq2slider",
        width=120,
        min=0.001,
        max=SAMPLE_MAX_FREQ,
        value=SAMPLE_FREQUENCY_2,
        notifier = function(freq)
          SAMPLE_FREQUENCY_2 = tonumber(freq)
          vb.views.freq2box.value = tonumber(freq)
          redraw_sample()
        end
      },
      vb:valuefield{
        id="freq2box",
        width=50,
        min=0.001,
        max=SAMPLE_MAX_FREQ,
        value=SAMPLE_FREQUENCY_2,
        notifier = function(freq)
          SAMPLE_FREQUENCY_2 = tonumber(freq)
          vb.views.freq2slider.value = tonumber(freq)
          redraw_sample()
        end
      },
    },
    -- ----------------------------------------- amplitude OSC 2
    vb:row{
      vb:text{text="Amplitude", align="left", width=80},
      vb:minislider{
        height=MINISLIDER_HEIGHT,
        id = "amp2slider",
        width=120,
        min=SAMPLE_MIN_AMPLITUDE,
        max=SAMPLE_MAX_AMPLITUDE,
        value=SAMPLE_AMPLITUDE_2,
        notifier = function(amplitude)
          SAMPLE_AMPLITUDE_2 = tonumber(amplitude)
          vb.views.amp2box.value = tonumber(amplitude)
          redraw_sample()
        end
      },
      vb:valuefield{
        id="amp2box",
        width=50,
        min=SAMPLE_MIN_AMPLITUDE,
        max=SAMPLE_MAX_AMPLITUDE,
        value=SAMPLE_AMPLITUDE_2,
        notifier = function(amplitude)
          SAMPLE_AMPLITUDE_2 = tonumber(amplitude)
          vb.views.amp2slider.value = tonumber(amplitude)
          redraw_sample()
        end
      },
    },
    -- ------------------------------------------- phase OSC 2
    vb:row{
      vb:text{text="Phase", align="left", width=80},
      vb:minislider{
        height=MINISLIDER_HEIGHT,
        id = "phase2slider",
        width=120,
        min=-SAMPLE_MAX_PHASE,
        max=SAMPLE_MAX_PHASE,
        value=SAMPLE_PHASE_1,
        notifier = function(phase)
          SAMPLE_PHASE_2 = tonumber(phase)
          vb.views.phase2box.value = tonumber(phase)
          redraw_sample()
        end
      },
      vb:valuefield{
        id="phase2box",
        width=50,
        min=-SAMPLE_MAX_PHASE,
        max=SAMPLE_MAX_PHASE,
        value=SAMPLE_PHASE_2,
        notifier = function(phase)
          SAMPLE_PHASE_2 = tonumber(phase)
          vb.views.phase2slider.value = tonumber(phase)
          redraw_sample()
        end
      },
    },
  }

--[[----------------------------------------------------------------
Create OSC 3 Settings                                              -
]]------------------------------------------------------------------
  local osc_3_gui = vb:column{
    style="group",
    margin=DEFAULT_MARGIN,
    vb:horizontal_aligner{
      mode="left",
      vb:text{
        text="Xochiquetzal",
        font="bold",
        style="strong",
      },
    },
    -- -------------------------------------------frequency OSC 3
    vb:row{
      vb:text{text="Frequency", align="left", width=80},
      vb:minislider{
        height=MINISLIDER_HEIGHT,
        id = "freq3slider",
        width=120,
        min=0.001,
        max=SAMPLE_MAX_FREQ,
        value=SAMPLE_FREQUENCY_3,
        notifier = function(freq)
          SAMPLE_FREQUENCY_3 = freq
          vb.views.freq3box.value = freq
          redraw_sample()
        end
      },
      vb:valuefield{
        id="freq3box",
        width=50,
        min=0.001,
        max=SAMPLE_MAX_FREQ,
        value=SAMPLE_FREQUENCY_3,
        notifier = function(freq)
          SAMPLE_FREQUENCY_3 = freq
          vb.views.freq3slider.value = tonumber(freq)
          redraw_sample()
        end
      },
    },
    -- -------------------------------------------- amplitude OSC 3
    vb:row{
      vb:text{text="Amplitude", align="left", width=80},
      vb:minislider{
        height=MINISLIDER_HEIGHT,
        id = "amp3slider",
        width=120,
        min=SAMPLE_MIN_AMPLITUDE,
        max=SAMPLE_MAX_AMPLITUDE,
        value=SAMPLE_AMPLITUDE_3,
        notifier = function(amplitude)
          SAMPLE_AMPLITUDE_3 = amplitude
          vb.views.amp3box.value = amplitude
          redraw_sample()
        end
      },
      vb:valuefield{
        id="amp3box",
        width=50,
        min=SAMPLE_MIN_AMPLITUDE,
        max=SAMPLE_MAX_AMPLITUDE,
        value=SAMPLE_AMPLITUDE_3,
        notifier = function(amplitude)
          SAMPLE_AMPLITUDE_3 = amplitude
          vb.views.amp3slider.value = tonumber(amplitude)
          redraw_sample()
        end
      },
    },
    -- -------------------------------------------- phase OSC 3
    vb:row{
      vb:text{text="Phase", align="left", width=80},
      vb:minislider{
        height=MINISLIDER_HEIGHT,
        id = "phase3slider",
        width=120,
        min=-SAMPLE_MAX_PHASE,
        max=SAMPLE_MAX_PHASE,
        value=SAMPLE_PHASE_3,
        notifier = function(phase)
          SAMPLE_PHASE_3 = phase
          vb.views.phase3box.value = phase
          redraw_sample()
        end
      },
      vb:valuefield{
        id="phase3box",
        width=50,
        min=-SAMPLE_MAX_PHASE,
        max=SAMPLE_MAX_PHASE,
        value=SAMPLE_PHASE_3,
        notifier = function(phase)
          SAMPLE_PHASE_3 = phase
          vb.views.phase3slider.value = tonumber(phase)
          redraw_sample()
        end
      },
    },
  }
      
--[[-------------------------------------------------------------------------------
Noisy Controls                                                                    -
]]---------------------------------------------------------------------------------
  local noisy_gui = vb:column{
    style="group",
    margin=DEFAULT_MARGIN,
    width=258,
    vb:row{
      vb:horizontal_aligner{
        mode="center",
        vb:text{
          width=200,
          id="noisy_message",
          text="Sak Nik sleeps",
          font="bold",
          style="strong",
        },
      },
    },
    vb:row{
      vb:text{text="Amount", align="left", width=80},
      vb:minislider{
        height=MINISLIDER_HEIGHT,
        width=160,
        id="noise",
        min=0,
        max=SAMPLE_MAX_NOISE,
        value=SAMPLE_NOISE_AMOUNT,
        notifier=function(noise)
          SAMPLE_NOISE_AMOUNT = noise
          if noise < 0.1 then
            SAMPLE_NOISE_BOOL = false
            vb.views.noisy_message.text="Sak Nik sleeps"
          else
            SAMPLE_NOISE_BOOL = true
            vb.views.noisy_message.text="Sak Nik is awake"
          end
          redraw_sample()
        end
      },
    },
  }

--[[------------------------------------------------------------------------------------
Sample Frames control                                                                  -
]]--------------------------------------------------------------------------------------
  local frames_gui = vb:column{
    style="group",
    margin=DEFAULT_MARGIN,
    vb:row{
      vb:text{text="Sample frames", align="left", width=80},
      vb:minislider{
        height=MINISLIDER_HEIGHT,
        id = "sample_frames_slider",
        width=120,
        min=8,
        max=math.floor(SAMPLE_FREQUENCY / 4),
        value=SAMPLE_FRAMES,
        notifier = function(sample_frames)
          SAMPLE_FRAMES = math.floor(sample_frames) - (math.floor(SAMPLE_FRAMES) % 2)
          vb.views.sample_frames_box.value = sample_frames
          redraw_sample()
        end
      },
      vb:valuefield{
        id="sample_frames_box",
        width=50,
        min=8,
        max=math.floor(SAMPLE_FREQUENCY / 4),
        value=SAMPLE_FRAMES,
        notifier = function(sample_frames)
          SAMPLE_FRAMES = math.floor(sample_frames) - (math.floor(sample_frames) % 2)
          redraw_sample()
        end
      },
    },
  }

--[[--------------------------------------------------------------------------------------
Operation control                                                                        -
]]----------------------------------------------------------------------------------------
  local operator_gui = vb:column{
    style="group",
    margin=DEFAULT_MARGIN,
    width=262,
    vb:horizontal_aligner{
      mode="center",
      vb:row{
        spacing=3,
        vb:text{text="Op1->", align="left"},
        vb:popup{
          id="op1",
          width=70,
          items=OPERATIONS,
          value=1,
          notifier=function(value)
            OP1=tonumber(value)
            redraw_sample()
          end
        },
        vb:text{text="Op2->", align="left"},
        vb:popup{
          id="op2",
          width=70,
          items=OPERATIONS,
          value = 1,
          notifier=function(value)
            OP2=tonumber(value)
            redraw_sample()
          end
        },
        vb:text{text="Op3", align="left"},
      },
    },
  }

--[[--------------------------------------------------------------------------------------
GAUSSIAN CONTROL                                                                         -
]]----------------------------------------------------------------------------------------
  local gaussian_gui = vb:column{
    spacing=DEFAULT_SPACING,
    margin = DEFAULT_MARGIN,
    height = DEFAULT_HEIGHT,
    vb:row{
        vb:checkbox{
          id="gaussian_on_off",
          value=GAUSSIAN_ON_OFF,
          notifier=function(bool)
            GAUSSIAN_ON_OFF = bool
            redraw_sample()
          end
        },
        vb:text{
          style="strong",
          font="bold",
          text="Let the gods work together",
          width=130,
        },
      },
    vb:column{
      id="gods1",
      spacing=DEFAULT_SPACING,
      margin = DEFAULT_MARGIN,
      height = DEFAULT_HEIGHT,
      style="group",
      vb:vertical_aligner{
        mode="center",
        vb:row{
          vb:text{text="Huehuecóyotl rising", width=130},
          vb:minislider{
            id="gauss_v1",
            height=MINISLIDER_HEIGHT,
            width=120,
            min=GAUSSIAN_V_MIN,
            max=GAUSSIAN_V_MAX,
            value=GAUSSIAN_V1,
            notifier = function(value)
              GAUSSIAN_V1 = value
              redraw_sample()
            end
          },
        },
        vb:row{
          vb:text{text="Ah-Xoc-Xin rising", width=130},
          vb:minislider{
            id="gauss_v2",
            height=MINISLIDER_HEIGHT,
            width=120,
            min=GAUSSIAN_V_MIN,
            max=GAUSSIAN_V_MAX,
            value=GAUSSIAN_V2,
            notifier = function(value)
              GAUSSIAN_V2 = value
              redraw_sample()
            end
          },
        },
        vb:row{
          vb:text{text="Xochiquetzal rising", width=130},
          vb:minislider{
            id="gauss_v3",
            height=MINISLIDER_HEIGHT,
            width=120,
            min=GAUSSIAN_V_MIN,
            max=GAUSSIAN_V_MAX,
            value=GAUSSIAN_V3,
            notifier = function(value)
              GAUSSIAN_V3 = value
              redraw_sample()
            end
          },
        },
        vb:row{
          vb:text{text="Sak Nik rising", width=130},
          vb:minislider{
            id="gauss_v4",
            height=MINISLIDER_HEIGHT,
            width=120,
            min=GAUSSIAN_V_MIN,
            max=GAUSSIAN_V_MAX,
            value=GAUSSIAN_V4,
            notifier = function(value)
              GAUSSIAN_V4 = value
              redraw_sample()
            end
          },
        },
        vb:row{
          vb:text{text="Macuilxochitl rising", width=130},
          vb:minislider{
            id="gauss_v5",
            height=MINISLIDER_HEIGHT,
            width=120,
            min=GAUSSIAN_V_MIN,
            max=GAUSSIAN_V_MAX,
            value=GAUSSIAN_V5,
            notifier = function(value)
              GAUSSIAN_V5 = value
              redraw_sample()
            end
          },
        },
      },
    },
    vb:column{
      id="gods2",
      spacing=DEFAULT_SPACING,
      margin = DEFAULT_MARGIN,
      height = DEFAULT_HEIGHT,
      style="group",
      vb:row{
        vb:text{text="Huehuecóyotl dancing", width=130},
        vb:minislider{
          id="gauss_n1",
          height=MINISLIDER_HEIGHT,
          width=120,
          min=GAUSSIAN_N_MIN,
          max=GAUSSIAN_N_MAX,
          value=GAUSSIAN_N1,
          notifier = function(value)
            GAUSSIAN_N1 = value
            redraw_sample()
          end
        },
      },
      vb:row{
        vb:text{text="Ah-Xoc-Xin dancing", width=130},
        vb:minislider{
          id="gauss_n2",
          height=MINISLIDER_HEIGHT,
          width=120,
          min=GAUSSIAN_N_MIN,
          max=GAUSSIAN_N_MAX,
          value=GAUSSIAN_N2,
          notifier = function(value)
            GAUSSIAN_N2 = value
            redraw_sample()
          end
        },
      },
      vb:row{
        vb:text{text="Xochiquetzal dancing", width=130},
        vb:minislider{
          id="gauss_n3",
          height=MINISLIDER_HEIGHT,
          width=120,
          min=GAUSSIAN_N_MIN,
          max=GAUSSIAN_N_MAX,
          value=GAUSSIAN_N3,
          notifier = function(value)
            GAUSSIAN_N3 = value
            redraw_sample()
          end
        },
      },
      vb:row{
        vb:text{text="Sak Nik dancing", width=130},
        vb:minislider{
          id="gauss_n4",
          height=MINISLIDER_HEIGHT,
          width=120,
          min=GAUSSIAN_N_MIN,
          max=GAUSSIAN_N_MAX,
          value=GAUSSIAN_N4,
          notifier = function(value)
            GAUSSIAN_N4 = value
            redraw_sample()
          end
        },
      },
      vb:row{
        vb:text{text="Macuilxochitl dancing", width=130},
        vb:minislider{
          id="gauss_n5",
          height=MINISLIDER_HEIGHT,
          width=120,
          min=GAUSSIAN_N_MIN,
          max=GAUSSIAN_N_MAX,
          value=GAUSSIAN_N5,
          notifier = function(value)
            GAUSSIAN_N5 = value
            redraw_sample()
          end
        },
      },
    },
    vb:column{
      id="gods3",
      spacing=DEFAULT_SPACING,
      margin = DEFAULT_MARGIN,
      height = DEFAULT_HEIGHT,
      style="group",
      vb:row{
        vb:text{text="Huehuecóyotl expands", width=130},
        vb:minislider{
          id="gauss_s1",
          height=MINISLIDER_HEIGHT,
          width=120,
          min=GAUSSIAN_S_MIN,
          max=GAUSSIAN_S_MAX,
          value=GAUSSIAN_S1,
          notifier = function(value)
            GAUSSIAN_S1 = value
            redraw_sample()
          end
        },
      },
      vb:row{
        vb:text{text="Ah-Xoc-Xin expands", width=130},
        vb:minislider{
          id="gauss_s2",
          height=MINISLIDER_HEIGHT,
          width=120,
          min=GAUSSIAN_S_MIN,
          max=GAUSSIAN_S_MAX,
          value=GAUSSIAN_S2,
          notifier = function(value)
            GAUSSIAN_S2 = value
            redraw_sample()
          end
        },
      },
      vb:row{
        vb:text{text="Xochiquetzal expands", width=130},
        vb:minislider{
          id="gauss_s3",
          height=MINISLIDER_HEIGHT,
          width=120,
          min=GAUSSIAN_S_MIN,
          max=GAUSSIAN_S_MAX,
          value=GAUSSIAN_S3,
          notifier = function(value)
            GAUSSIAN_S3 = value
            redraw_sample()
          end
        },
      },
      vb:row{
        vb:text{text="Sak Nik expands", width=130},
        vb:minislider{
          id="gauss_s4",
          height=MINISLIDER_HEIGHT,
          width=120,
          min=GAUSSIAN_S_MIN,
          max=GAUSSIAN_S_MAX,
          value=GAUSSIAN_S4,
          notifier = function(value)
            GAUSSIAN_S4 = value
            redraw_sample()
          end
        },
      },
      vb:row{
        vb:text{text="Macuilxochitl expands", width=130},
        vb:minislider{
          id="gauss_s5",
          height=MINISLIDER_HEIGHT,
          width=120,
          min=GAUSSIAN_S_MIN,
          max=GAUSSIAN_S_MAX,
          value=GAUSSIAN_S5,
          notifier = function(value)
            GAUSSIAN_S5 = value
            redraw_sample()
          end
        },
      },
    },
  }

  --[[------------------------------------------------------------------------
  Patches page                                                               -
  ]]--------------------------------------------------------------------------
  local patches_gui = vb:column{
    spacing=DEFAULT_SPACING,
    margin = DEFAULT_MARGIN,
    height = DEFAULT_HEIGHT,
    width=264,
    style="group",
    uniform=true,
    vb:horizontal_aligner{
      mode="center",
      vb:text{
        text="please note that"
      },
    },
    vb:horizontal_aligner{
      mode="center",
      vb:text{
        text="this is an experimental version only.",
      },
    },
    vb:horizontal_aligner{
      mode="center",
      vb:text{
        text="All settings will definitely change",
      },
    },
    vb:horizontal_aligner{
      mode="center",
      vb:text{
        text="in the next version.",
      },
    },
    vb:space{height=10},
    vb:horizontal_aligner{
      mode="center",
      vb:button{
        text="Load patch",
        width=200,
        notifier=function()
          local filename = renoise.app():prompt_for_filename_to_read({"*.mxp"}, "Load Macuilxochitl patch")
          if #filename ~= 0 then
            load_patch_file(filename)
            redraw_sample()
          end
        end
      },
    },
    vb:space{height=10},
    vb:horizontal_aligner{
      mode="center",
      vb:text{
        text="Keep in mind that"
      },
    },
    vb:horizontal_aligner{
      mode="center",
      vb:text{
        text="this extension is just my two day",
      },
    },
    vb:horizontal_aligner{
      mode="center",
      vb:text{
        text="acquaintance with Renoise Api",
      },
    },
    vb:horizontal_aligner{
      mode="center",
      vb:text{
        text="and Lua in general. Thanks",
      },
    },
    vb:space{height=10},
    vb:horizontal_aligner{
      mode="center",
      vb:button{
        width=200,
        text="Save patch",
        notifier=function()
          local filename = renoise.app():prompt_for_filename_to_write("mxp", "Save Macuilxochitl patch")
          if #filename ~= 0 then
            save_patch_file(filename)
          end
        end
      }
    },
  }

  --[[------------------------------------------------------------------------
  Some little info                                                           -
  ]]--------------------------------------------------------------------------
  local info_page = vb:column{
    spacing=DEFAULT_SPACING,
    margin = DEFAULT_MARGIN,
    height = DEFAULT_HEIGHT,
    style="group",
    vb:bitmap{
      mode="transparent",
      bitmap="gfx/info.bmp",
    },
    vb:horizontal_aligner{
      mode="center",
      vb:vertical_aligner{
        mode="top",
        vb:horizontal_aligner{
          mode="center",
          vb:text{
            font="bold",
            style="strong",
            align="center",
            text="Macuilxochitl"
          },
        },
        vb:horizontal_aligner{
          mode="center",
          vb:text{
            font="italic",
            align="center",
            text="Is the Mesoamerican god of music and dance",
          },
        },
        vb:horizontal_aligner{
          mode="center",
          vb:text{
            align="center",
            font="italic",
            text="and an experimental extension for Renoise.",
          },
        },
        vb:horizontal_aligner{
          mode="center",
          vb:text{
            style="strong",
            font="italic",
            align="center",
            text="Please be careful and protect your hearing !",
          },
        },
        vb:horizontal_aligner{
          mode="center",
          vb:text{
            font="italic",
            align="center",
            text="Some of the gods are still quite unstable",
          },
        },
        vb:horizontal_aligner{
          mode="center",
          vb:text{
            font="italic",
            align="center",
            text="because they are trapped in an unstable code.",
          },
        },
        vb:horizontal_aligner{
          mode="center",
          vb:text{
            font="italic",
            align="center",
            text="If you like the extension, use it. Thanks",
          },
        },
        vb:horizontal_aligner{
          mode="center",
          vb:button{
            width=200,
            height=16,
            text="I understand",
            notifier=function()
              vb.views.switch.visible=true
              vb.views.page1.visible=true
              vb.views.page2.visible=false
              vb.views.page3.visible=false
              vb.views.page4.visible=false
              vb.views.logo_bitmap.visible=true
              SELECTED_PAGE = 1
              redraw_sample()
            end
          }
        }
      },
    },
  }

  --[[------------------------------------------------------------------------
  Main gui                                                                   -
  ]]--------------------------------------------------------------------------
  local make_gui = vb:column{
    spacing=DEFAULT_SPACING,
    margin=DEFAULT_MARGIN,
    height=DEFAULT_HEIGHT,
    vb:horizontal_aligner{
      id="switch",
      mode="center",
      vb:row{
        vb:button{
          width=MENU_BUTTON_WIDTH,
          height=MENU_BUTTON_HEIGHT,
          text="Waves",
          color={30,50,30},
          notifier = function()
            vb.views.page1.visible = true
            vb.views.page2.visible = false
            vb.views.page3.visible = false
            vb.views.page4.visible = false
          end
        },
        vb:button{
          width=MENU_BUTTON_WIDTH,
          height=MENU_BUTTON_HEIGHT,
          text="Gods",
          color={30,50,30},
          notifier = function()
            vb.views.page1.visible = false
            vb.views.page2.visible = true
            vb.views.page3.visible = false
            vb.views.page4.visible = false
          end
        },
        vb:button{
          width=MENU_BUTTON_WIDTH,
          height=MENU_BUTTON_HEIGHT,
          text="Patches",
          color={30,50,30},
          notifier = function()
            vb.views.page1.visible = false
            vb.views.page2.visible = false
            vb.views.page3.visible = true
            vb.views.page4.visible = false
          end
        },
        vb:button{
          width=MENU_BUTTON_WIDTH,
          height=MENU_BUTTON_HEIGHT,
          text="Info",
          color={30,50,30},
          notifier = function()
            vb.views.page1.visible = false
            vb.views.page2.visible = false
            vb.views.page3.visible = false
            vb.views.page4.visible = true
          end
        },
      }
    },
    vb:column{
      spacing=DEFAULT_SPACING,
      margin = DEFAULT_MARGIN,
      height = DEFAULT_HEIGHT,
      id="page1",
      osc_1_gui,
      osc_2_gui,
      osc_3_gui,
      noisy_gui,
      frames_gui,
      operator_gui,
    },
    vb:column{
      id="page2",
      gaussian_gui,
    },
    vb:column{
      id="page4",
      info_page,
    },
    vb:column{
      id="page3",
      patches_gui,
    },
    vb:bitmap{
      id="logo_bitmap",
      mode="transparent",
      bitmap="gfx/logo.bmp"
    },
  }

  vb.views.switch.visible=false
  vb.views.page1.visible=false
  vb.views.page2.visible=false
  vb.views.page3.visible=false
  vb.views.page4.visible=true
  vb.views.logo_bitmap.visible=false
  
  dialog = renoise.app():show_custom_dialog(
    "Macuilxochitl v"..VERSION,
    make_gui)

end
